//--------------------------------------------------------------
// Press F1 to get help about using script.
// To access an object that is not located in the current class, start the call with Globals.
// When using events and timers be cautious not to generate memoryleaks,
// please see the help for more information.
//---------------------------------------------------------------

namespace Neo.ApplicationFramework.Generated
{
    using System.Windows.Forms;
    using System;
    using System.Drawing;
    using Neo.ApplicationFramework.Tools;
    using Neo.ApplicationFramework.Common.Graphics.Logic;
    using Neo.ApplicationFramework.Controls;
    using Neo.ApplicationFramework.Interfaces;
        
	using Neo.ApplicationFramework.Tools.OpcClient;
	using System.IO.Ports;   
	using System.Text;
	using System.Reflection;
	
    public partial class SerialComScript
	{
		public static SerialCom serialCom = null;
		
		static SerialComScript() 
		{
			serialCom = new SerialCom();
		}
		
		public void Send(string data)
		{
			serialCom.SendData(data);
		}

		public static void Dbg(string data)
		{
			//Globals.Tags.Tag1.Value = Globals.Tags.Tag1.Value + data;	
		}
    }
	
	public class SerialCom
	{
		public SerialPort sPort = null;
		
		private Timer myTimer = new Timer();
		
		public SerialCom()
		{
			try {
				sPort = new SerialPort("COM5", 9600, Parity.None, 8, StopBits.One);
				sPort.Open();
			}
			catch (Exception ex) {
				MessageBox.Show("Serial Port Open Fail, " + ex.Message);
			}			
			
			myTimer.Tick += new EventHandler(TimerEventProcessor);
			myTimer.Interval = 100;
			myTimer.Enabled = true;
		}

		public void SendData(string outData) {
			sPort.Write(outData);
		}
		
		StringBuilder buffer = new StringBuilder();
				
		private void TimerEventProcessor(Object myObject, EventArgs myEventArgs) 
		{			
			string dataStr = "";
			
			try {
				dataStr = sPort.ReadExisting();
			}
			catch (Exception) { return; }
			if (dataStr.Length == 0) { return; }
						
			ParseSerialProtocol(ref buffer, ref dataStr);
			
			//Put Code here to Handle Received Serial Data
			//SerialComScript.Dbg(" " + tmp);
		}		
		
		/// <summary>
		/// If true quotation marks will be automatically removed from incoming
		/// value strings.
		/// </summary>
		public bool TrimQuotes = true;
		
		private void ParseSerialProtocol(ref StringBuilder buff, ref string dataStr)
		{
			buff.Append(dataStr);
			// find ;
			int idx1 = buff.ToString().IndexOf(';');
			while (idx1 >= 0) {
				string objName = "";
				string propName = "";
				string value = "";

				string packet = buff.ToString(0, idx1);
				buff = buff.Remove(0, idx1 + 1);

				// find =
				idx1 = packet.IndexOf('=');
				bool isEqFound = false;
				if (idx1 >= 0) {
					isEqFound = true;
					value = packet.Substring(idx1 + 1);
					packet = packet.Substring(0, idx1);
				}

				// find .
				idx1 = packet.IndexOf('.');
				if (idx1 >= 0) {
					objName = packet.Substring(0, idx1);
					propName = packet.Substring(idx1 + 1);
				}
				else {
					propName = packet;
				}

				if (TrimQuotes) {
					char[] removeChars = new char[] { '\"' };
					value = value.Trim(removeChars);
				}

				if (isEqFound) {
					OnDataRecieved(ref objName, ref propName, ref value);
				}
				else {
					string data = OnDataRecieved(ref objName, ref propName);
					SendData(data);
				}

				// find ;
				idx1 = buff.ToString().IndexOf(';');
			}

			// keep within reasonable amount of space
			if (buff.Length > 100000) { buff = new StringBuilder(); }
		}
		
		/// <summary>
		/// This function is called when data is received in the format of objname.propname=value;.
		/// If packet is in the format of propName=value; then the objName string is empty.
		/// </summary>
		/// <param name="objName">string called objName that is in a packet in the format
		/// of objName.propName=value;</param>
		/// <param name="propName">string called propName that is in a packet in the format of
		/// objName.propName=value; or propName=value;</param>
		/// <param name="value">string called value that is in a packet in the format of
		/// objName.propName=value; or propName=value;</param>
		public void OnDataRecieved(ref string objName, ref string propName, ref string value)
		{
			GlobalDataItem tag = StringToTag(propName);
			if (tag != null) {
				try {
					if (value.IndexOf('.') >= 0) {
						Double d = Convert.ToDouble(value);
						tag.SetAnalog(d);
					}
					else {
						Int64 i = Convert.ToInt64(value);
						tag.SetAnalog(i);
					}
				}
				catch (Exception) {					
					tag.SetString(value);
				}
			}				
		}

		/// <summary>
		///  This function is called when data is received in the format of objname.propname;
		///  the return value is transmitted in reply to this packet. Return an empty string
		///  in order to not reply. If incoming data has the format of propName; then
		///  objName is empty.
		/// </summary>
		/// <param name="objName">string called objName in a packet that is in the format of
		/// objName.propName;</param>
		/// <param name="propName">string called propName in a packet that is in the format of
		/// objName.propName; or propName;</param>
		/// <returns>Return string to be sent in reply to recieved object/property name</returns>
		public string OnDataRecieved(ref string objName, ref string propName) 
		{
			GlobalDataItem tag = StringToTag(propName);
			if (tag != null) {
				return tag.Value.String;
			}
			return "";
		}
				
		// returns null if tag not found
		public GlobalDataItem StringToTag(string tagName)
		{
			string typeName = (new GlobalDataItem()).GetType().Name;
			PropertyInfo[] props = Globals.Tags.GetType().GetProperties();
			foreach (PropertyInfo prop in props) {
				if (prop.PropertyType.Name.Equals(typeName)) {
					if (prop.Name.Equals(tagName, StringComparison.CurrentCultureIgnoreCase)) {
						return (GlobalDataItem)prop.GetValue(Globals.Tags, null);
					}					
				}
			}
			return null;
		}	
	}
}
